# -*- coding: UTF-8 -*-
"""
  Author:  Jacek Kotlarski --<szumak@virthost.pl>
  Created: 03.11.2023

  Purpose: Base class for classess derived from threading.Thread
"""

from inspect import currentframe
from typing import Any, Optional, Tuple, Dict
from threading import Event
from jsktoolbox.attribtool import NoDynamicAttributes, ReadOnlyClass
from jsktoolbox.raisetool import Raise
from jsktoolbox.libs.base_data import BData


class _Keys(object, metaclass=ReadOnlyClass):
    """Keys definition class.

    For internal purpose only.
    """

    ARGS = "_args"
    DAEMONIC = "_daemonic"
    DEBUG = "_debug"
    IDENT = "_ident"
    INVOKE_EXCEPTHOOK = "_invoke_excepthook"
    IS_STOPPED = "_is_stopped"
    KWARGS = "_kwargs"
    NAME = "_name"
    NATIVE_ID = "_native_id"
    SLEEP_PERIOD = "_sleep_period"
    STARTED = "_started"
    STDERR = "_stderr"
    STOP_EVENT = "_stop_event"
    TARGET = "_target"
    TSTATE_LOCK = "_tstate_lock"


class ThBaseObject(BData, NoDynamicAttributes):
    """Base class for classes derived from threading.Thread.

    Definition of properties used in the threading library.
    """

    @property
    def _target(self) -> Optional[Any]:
        if _Keys.TARGET not in self._data:
            self._data[_Keys.TARGET] = None
        return self._data[_Keys.TARGET]

    @_target.setter
    def _target(self, value: Any) -> None:
        self._data[_Keys.TARGET] = value

    @property
    def _name(self) -> Optional[str]:
        if _Keys.NAME not in self._data:
            self._data[_Keys.NAME] = None
        return self._data[_Keys.NAME]

    @_name.setter
    def _name(self, value: Optional[str]) -> None:
        if value is not None and not isinstance(value, str):
            raise Raise.error(
                f"String type expected, '{type(value)}' received.",
                TypeError,
                self.__class__.__name__,
                currentframe(),
            )
        self._data[_Keys.NAME] = value

    @property
    def _args(self) -> Optional[Tuple]:
        if _Keys.ARGS not in self._data:
            self._data[_Keys.ARGS] = None
        return self._data[_Keys.ARGS]

    @_args.setter
    def _args(self, value: Tuple) -> None:
        self._data[_Keys.ARGS] = value

    @property
    def _kwargs(self) -> Optional[Dict]:
        if _Keys.KWARGS not in self._data:
            self._data[_Keys.KWARGS] = None
        return self._data[_Keys.KWARGS]

    @_kwargs.setter
    def _kwargs(self, value: Dict) -> None:
        if value is not None and not isinstance(value, Dict):
            raise Raise.error(
                f"Dict type expected, '{type(value)}' received.",
                TypeError,
                self.__class__.__name__,
                currentframe(),
            )
        self._data[_Keys.KWARGS] = value

    @property
    def _daemonic(self) -> Optional[bool]:
        if _Keys.DAEMONIC not in self._data:
            self._data[_Keys.DAEMONIC] = None
        return self._data[_Keys.DAEMONIC]

    @_daemonic.setter
    def _daemonic(self, value: bool) -> None:
        if not isinstance(value, bool):
            raise Raise.error(
                f"Boolean type expected, '{type(value)}' received.",
                TypeError,
                self.__class__.__name__,
                currentframe(),
            )
        self._data[_Keys.DAEMONIC] = value

    @property
    def _debug(self) -> Optional[bool]:
        if _Keys.DEBUG not in self._data:
            self._data[_Keys.DEBUG] = None
        return self._data[_Keys.DEBUG]

    @_debug.setter
    def _debug(self, value: bool) -> None:
        if not isinstance(value, bool):
            raise Raise.error(
                f"Boolean type expected, '{type(value)}' received.",
                TypeError,
                self.__class__.__name__,
                currentframe(),
            )
        self._data[_Keys.DEBUG] = value

    @property
    def _ident(self) -> Optional[int]:
        if _Keys.IDENT not in self._data:
            self._data[_Keys.IDENT] = None
        return self._data[_Keys.IDENT]

    @_ident.setter
    def _ident(self, value: Optional[int]) -> None:
        if value is not None and not isinstance(value, int):
            raise Raise.error(
                f"Integer type expected, '{type(value)}' received.",
                TypeError,
                self.__class__.__name__,
                currentframe(),
            )
        self._data[_Keys.IDENT] = value

    @property
    def _native_id(self) -> Optional[int]:
        if _Keys.NATIVE_ID not in self._data:
            self._data[_Keys.NATIVE_ID] = None
        return self._data[_Keys.NATIVE_ID]

    @_native_id.setter
    def _native_id(self, value: Optional[int]) -> None:
        if value is not None and not isinstance(value, int):
            raise Raise.error(
                f"Integer type expected, '{type(value)}' received.",
                TypeError,
                self.__class__.__name__,
                currentframe(),
            )
        self._data[_Keys.NATIVE_ID] = value

    @property
    def _tstate_lock(self) -> Optional[Any]:
        if _Keys.TSTATE_LOCK not in self._data:
            self._data[_Keys.TSTATE_LOCK] = None
        return self._data[_Keys.TSTATE_LOCK]

    @_tstate_lock.setter
    def _tstate_lock(self, value: Any) -> None:
        self._data[_Keys.TSTATE_LOCK] = value

    @property
    def _started(self) -> Optional[Event]:
        if _Keys.STARTED not in self._data:
            self._data[_Keys.STARTED] = None
        return self._data[_Keys.STARTED]

    @_started.setter
    def _started(self, value: Event) -> None:
        if value is not None and not isinstance(value, Event):
            raise Raise.error(
                f"threading.Event type expected, '{type(value)}' received.",
                TypeError,
                self.__class__.__name__,
                currentframe(),
            )
        self._data[_Keys.STARTED] = value

    @property
    def _is_stopped(self) -> Optional[bool]:
        if _Keys.IS_STOPPED not in self._data:
            self._data[_Keys.IS_STOPPED] = None
        return self._data[_Keys.IS_STOPPED]

    @_is_stopped.setter
    def _is_stopped(self, value: bool) -> None:
        if not isinstance(value, bool):
            raise Raise.error(
                f"Boolean type expected, '{type(value)}' received.",
                TypeError,
                self.__class__.__name__,
                currentframe(),
            )
        self._data[_Keys.IS_STOPPED] = value

    @property
    def _stderr(self) -> Optional[Any]:
        if _Keys.STDERR not in self._data:
            self._data[_Keys.STDERR] = None
        return self._data[_Keys.STDERR]

    @_stderr.setter
    def _stderr(self, value: Any) -> None:
        self._data[_Keys.STDERR] = value

    @property
    def _invoke_excepthook(self) -> Optional[Any]:
        if _Keys.INVOKE_EXCEPTHOOK not in self._data:
            self._data[_Keys.INVOKE_EXCEPTHOOK] = None
        return self._data[_Keys.INVOKE_EXCEPTHOOK]

    @_invoke_excepthook.setter
    def _invoke_excepthook(self, value: Any) -> None:
        self._data[_Keys.INVOKE_EXCEPTHOOK] = value

    @property
    def _stop_event(self) -> Optional[Event]:
        if _Keys.STOP_EVENT not in self._data:
            self._data[_Keys.STOP_EVENT] = None
        return self._data[_Keys.STOP_EVENT]

    @_stop_event.setter
    def _stop_event(self, obj: Event) -> None:
        if obj is not None and not isinstance(obj, Event):
            raise Raise.error(
                f"threading.Event type expected, '{type(obj)}' received.",
                TypeError,
                self.__class__.__name__,
                currentframe(),
            )
        self._data[_Keys.STOP_EVENT] = obj

    @property
    def is_stopped(self) -> Optional[bool]:
        return self._is_stopped

    @property
    def started(self) -> bool:
        return self._started.is_set()

    @property
    def sleep_period(self) -> float:
        """Return sleepperiod value."""
        if _Keys.SLEEP_PERIOD not in self._data:
            self._data[_Keys.SLEEP_PERIOD] = 1.0
        return self._data[_Keys.SLEEP_PERIOD]

    @sleep_period.setter
    def sleep_period(self, value: float) -> None:
        """Set sleepperiod value."""
        if not isinstance(value, float):
            raise Raise.error(
                f"Positive float type expected, '{value}' received.",
                TypeError,
                self.__class__.__name__,
                currentframe(),
            )
        self._data[_Keys.SLEEP_PERIOD] = value


# #[EOF]#######################################################################



# -*- coding: UTF-8 -*-
"""
  Author:  Jacek Kotlarski --<szumak@virthost.pl>
  Created: 04.09.2023

  Purpose: base class for log subsystem.
"""

import syslog

from inspect import currentframe
from typing import Optional, Tuple, List, Dict, Any

from jsktoolbox.attribtool import NoDynamicAttributes, ReadOnlyClass
from jsktoolbox.raisetool import Raise
from jsktoolbox.libs.base_data import BData


class Keys(object, metaclass=ReadOnlyClass):
    """Keys definition class.

    For internal purpose only.
    """

    BUFFERED = "__buffered__"
    CONF = "__conf__"
    DIR = "__dir__"
    FILE = "__file__"
    FORMATTER = "__formatter__"
    FACILITY = "__facility__"
    LEVEL = "__level__"
    NAME = "__name__"
    NO_CONF = "__noconf__"
    QUEUE = "__queue__"
    SYSLOG = "__syslog__"


class SysLogKeys(object, metaclass=ReadOnlyClass):
    """SysLog keys definition container class."""

    class __Levels(object, metaclass=ReadOnlyClass):
        ALERT = syslog.LOG_ALERT
        CRITICAL = syslog.LOG_CRIT
        DEBUG = syslog.LOG_DEBUG
        EMERGENCY = syslog.LOG_EMERG
        ERROR = syslog.LOG_ERR
        INFO = syslog.LOG_INFO
        NOTICE = syslog.LOG_NOTICE
        WARNING = syslog.LOG_WARNING

    class __Facilities(object, metaclass=ReadOnlyClass):
        DAEMON = syslog.LOG_DAEMON
        USER = syslog.LOG_USER
        LOCAL0 = syslog.LOG_LOCAL0
        LOCAL1 = syslog.LOG_LOCAL1
        LOCAL2 = syslog.LOG_LOCAL2
        LOCAL3 = syslog.LOG_LOCAL3
        LOCAL4 = syslog.LOG_LOCAL4
        LOCAL5 = syslog.LOG_LOCAL5
        LOCAL6 = syslog.LOG_LOCAL6
        LOCAL7 = syslog.LOG_LOCAL7
        MAIL = syslog.LOG_MAIL
        SYSLOG = syslog.LOG_SYSLOG

    @classmethod
    @property
    def level(cls):
        """Returns Levels keys property."""
        return cls.__Levels

    @classmethod
    @property
    def facility(cls):
        """Returns Facility keys property."""
        return cls.__Facilities

    @classmethod
    @property
    def level_keys(cls) -> Dict:
        """Returns level keys property."""
        return {
            "NOTICE": SysLogKeys.level.NOTICE,
            "INFO": SysLogKeys.level.INFO,
            "DEBUG": SysLogKeys.level.DEBUG,
            "WARNING": SysLogKeys.level.WARNING,
            "ERROR": SysLogKeys.level.ERROR,
            "EMERGENCY": SysLogKeys.level.EMERGENCY,
            "ALERT": SysLogKeys.level.ALERT,
            "CRITICAL": SysLogKeys.level.CRITICAL,
        }

    @classmethod
    @property
    def facility_keys(cls) -> Dict:
        """Returns Facility keys property."""
        return {
            "DAEMON": SysLogKeys.facility.DAEMON,
            "USER": SysLogKeys.facility.USER,
            "LOCAL0": SysLogKeys.facility.LOCAL0,
            "LOCAL1": SysLogKeys.facility.LOCAL1,
            "LOCAL2": SysLogKeys.facility.LOCAL2,
            "LOCAL3": SysLogKeys.facility.LOCAL3,
            "LOCAL4": SysLogKeys.facility.LOCAL4,
            "LOCAL5": SysLogKeys.facility.LOCAL5,
            "LOCAL6": SysLogKeys.facility.LOCAL6,
            "LOCAL7": SysLogKeys.facility.LOCAL7,
            "MAIL": SysLogKeys.facility.MAIL,
            "SYSLOG": SysLogKeys.facility.SYSLOG,
        }


class LogsLevelKeys(object, metaclass=ReadOnlyClass):
    """LogsLevelKeys container class."""

    ALERT = "ALERT"
    CRITICAL = "CRITICAL"
    DEBUG = "DEBUG"
    EMERGENCY = "EMERGENCY"
    ERROR = "ERROR"
    INFO = "INFO"
    NOTICE = "NOTICE"
    WARNING = "WARNING"

    @classmethod
    @property
    def keys(cls) -> Tuple[str]:
        """Return tuple of avaiable keys."""
        return tuple(
            [
                LogsLevelKeys.ALERT,
                LogsLevelKeys.CRITICAL,
                LogsLevelKeys.DEBUG,
                LogsLevelKeys.EMERGENCY,
                LogsLevelKeys.ERROR,
                LogsLevelKeys.INFO,
                LogsLevelKeys.NOTICE,
                LogsLevelKeys.WARNING,
            ]
        )


class LoggerQueue(NoDynamicAttributes):
    """LoggerQueue simple class."""

    __queue: List[str] = None

    def __init__(self):
        """Constructor."""
        self.__queue = []

    def get(self) -> Optional[Tuple[str, str]]:
        """Get item from queue.

        Returs queue tuple[log_level:str, message:str] or None if empty.
        """
        try:
            return tuple(self.__queue.pop(0))
        except IndexError:
            return None
        except Exception as ex:
            raise Raise.error(
                f"Unexpected exception was thrown: {ex}",
                self.__class__.__name__,
                currentframe(),
            )

    def put(self, message: str, log_level: str = LogsLevelKeys.INFO) -> None:
        """Put item to queue."""
        if log_level not in LogsLevelKeys.keys:
            raise Raise.error(
                f"logs_level key not found, '{log_level}' received.",
                KeyError,
                self.__class__.__name__,
                currentframe(),
            )
        self.__queue.append(
            [
                log_level,
                message,
            ]
        )


class BLoggerQueue(BData, NoDynamicAttributes):
    """Logger Queue base metaclass."""

    @property
    def logs_queue(self) -> Optional[LoggerQueue]:
        """Get LoggerQueue object."""
        if Keys.QUEUE not in self._data:
            return None
        return self._data[Keys.QUEUE]

    @logs_queue.setter
    def logs_queue(self, obj: LoggerQueue) -> None:
        """Set LoggerQueue object."""
        if not isinstance(obj, LoggerQueue):
            raise Raise.error(
                f"LoggerQueue type object expected, '{type(obj)}' received.",
                self.__class__.__name__,
                currentframe(),
            )
        self._data[Keys.QUEUE] = obj


class BLoggerEngine(BData, NoDynamicAttributes):
    """Base class for LoggerEngine classes."""

    @property
    def name(self) -> Optional[str]:
        """Return app name string."""
        if Keys.NAME not in self._data:
            self._data[Keys.NAME] = None
        return self._data[Keys.NAME]

    @name.setter
    def name(self, value: str) -> None:
        """Set app name string."""
        self._data[Keys.NAME] = value


class BLogFormatter(NoDynamicAttributes):
    """Log formatter base class."""

    __template: Optional[str] = None
    __forms: Optional[List] = None

    def __init__(self) -> None:
        """Constructor."""
        self.__forms = []

    def format(self, message: str, name: str = None) -> str:
        """Method for format message string.

        Arguments:
        message [str]: log string to send
        name [str]: optional name of apps,
        """
        out = ""
        for item in self._forms_:
            if callable(item):
                out += f"{item()} "
            elif isinstance(item, str):
                if name is None:
                    if item.find("name") == -1:
                        out += item.format(message=f"{message}")
                else:
                    if item.find("name") > 0:
                        out += item.format(
                            name=f"{name}",
                            message=f"{message}",
                        )
        return out

    @property
    def _forms_(self) -> List:
        """Get forms list."""
        return self.__forms

    @_forms_.setter
    def _forms_(self, item: Any) -> None:
        """Set forms list."""
        # assigning function to a variable
        # def a(): print('test')
        # var=a
        # var()
        ####
        # >>> x._forms_[2].__class__
        # <class 'builtin_function_or_method'>
        # >>> x._forms_[1].__class__
        # <class 'float'>
        # >>> x._forms_[0].__class__
        # <class 'str'>

        self.__forms.append(item)


# #[EOF]#######################################################################



# -*- coding: UTF-8 -*-
"""
  Author:  Jacek Kotlarski --<szumak@virthost.pl>
  Created: 01.09.2023

  Purpose: BData container base class.
"""

import inspect
from typing import Dict, Optional
from jsktoolbox.attribtool import NoDynamicAttributes
from jsktoolbox.raisetool import Raise


class BData(NoDynamicAttributes):
    """BData container class."""

    __data: Optional[Dict] = None

    @property
    def _data(self) -> Dict:
        """Return data dict."""
        if self.__data is None:
            self.__data = {}
        return self.__data

    @_data.setter
    def _data(self, value: Optional[Dict]) -> None:
        """Set data dict."""
        if value is None:
            self.__data = {}
            return
        if isinstance(value, Dict):
            for key in value.keys():
                self.__data[key] = value[key]
        else:
            raise Raise.error(
                f"Dict type expected, '{type(value)}' received.",
                AttributeError,
                self.__class__.__name__,
                inspect.currentframe,
            )


# #[EOF]#######################################################################
